@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.6 2003/06/28 07:30:24 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/child.c,v 1.7 2003/07/05 06:32:33 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -185,27 +185,33 @@ void do_child(int unix_fd, int inet_fd)
 										connection_reuse);
 			switch (status)
 			{
-				case POOL_ERROR:
-					pool_discard_cp(sp->user, sp->database);
-					break;
-
+				/* client exits */
 				case POOL_END:
 					/* do not cache connection to template0, template1, regression */
 					if (!strcmp(sp->database, "template0") || !strcmp(sp->database, "template1") ||
 						!strcmp(sp->database, "regression"))
 						pool_discard_cp(sp->user, sp->database);
-					else
-						pool_close(frontend);
+
+					pool_close(frontend);
+					break;
+				
+				/* error occured. discard backend connection pool
+                   and disconnect connection to the frontend */
+				case POOL_ERROR:
+					pool_discard_cp(sp->user, sp->database);
+					pool_close(frontend);
 					break;
 
+				/* fatal error occured. just exit myself... */
+				case POOL_FATAL:
+					exit(1);
+					break;
+
+				/* not implemented yet */
 				case POOL_IDLE:
 					do_accept(unix_fd, inet_fd);
 					pool_debug("accept while idle");
 					break;
-
-				case POOL_FATAL:
-					exit(1);
-					break;
 			}
 
 			if (status != POOL_CONTINUE)
